#!/bin/bash

REPORT="/tmp/openMosix-stress-test-report-`date +%y%m%d%H%I`.txt"
STDOUTFILE="/tmp/openMosix-stress-test-output.txt"

# count the nodes in the openMosix cluster
HOWMANY=0 
for n in `ls /proc/hpc/nodes` 
do 
let "TMP=`cat /proc/hpc/nodes/$n/cpus`" 
if (( $TMP!="-101" )); 
then 
let "HOWMANY=HOWMANY+`cat /proc/hpc/nodes/$n/cpus`" 
fi 
done; 


echo | tee $STDOUTFILE
echo | tee -a $STDOUTFILE
echo "#######################################" | tee -a $STDOUTFILE
echo "starting the openMosix stress test now!" | tee -a $STDOUTFILE
echo "#######################################" | tee -a $STDOUTFILE
echo | tee -a $STDOUTFILE
echo "the results will be saved in : $REPORT" | tee -a $STDOUTFILE
echo | tee -a $STDOUTFILE

# check if /mfs is mounted
mount | grep /mfs
if [ $? == 1 ]
 then
 echo "oMFS is not mounted at /mfs! oMFS-test will be disabled." | tee -a $STDOUTFILE
 echo "Please mount oMFS before running this openMosix-test" | tee -a $STDOUTFILE
 echo "You will find instructions how to configure and use oMFS at:" | tee -a $STDOUTFILE
 echo "http://howto.ipng.be/openMosix-HOWTO/x222.htm#AEN243" | tee -a $STDOUTFILE
 echo ""
 OMFS=0
 else
 echo "oMFS is mounted at /mfs and will be tested" | tee -a $STDOUTFILE
 OMFS=1
fi


echo
echo "(return to continue, ctrl-c for cancel)"
read


# add description header
> $REPORT
echo "" >> $REPORT
echo "This are the results from the openMosix stress-test" >> $REPORT
echo "" >> $REPORT
echo "tested kernelversion : `uname -a`" >> $REPORT
echo "test was started on node : `mosctl whois | sed -e 's/.*#//'`" >> $REPORT
echo "cluster contains $HOWMANY nodes" >> $REPORT



echo "" >> $REPORT
echo "Timing from the application test" >> $REPORT
echo "" >> $REPORT


# unlock myself
echo 0 > /proc/self/lock

# start moving script on myself
ps ax > /tmp/pslist.txt
MYSELF=`cat /tmp/pslist.txt | grep "\.\/start_openMosix_test.sh" | awk '{ print $1 }'`

echo "my PID is $MYSELF" | tee -a $STDOUTFILE
echo "start moving.sh on myself" | tee -a $STDOUTFILE

moving/moving.sh $MYSELF >> $STDOUTFILE 2>/tmp/errormoving.txt &
sleep 4

ps ax > /tmp/pslist.txt
MOVING=`cat /tmp/pslist.txt | grep moving.sh | awk '{ print $1 }'`

# distkeygen
NOW=`date`
echo "starting distkeygen at $NOW"
echo "starting distkeygen at $NOW" >> $REPORT 
cd distkeygen
echo "output from distkeygen test" >> $STDOUTFILE
./start_distkeygen.sh >> $STDOUTFILE 2>/tmp/errordistkeygen.txt
#to be sure that no distkeygen is running any more
cat /tmp/errordistkeygen.txt
sleep 5
killall distkeygen
./cleanup.sh
cd ..
NOW=`date`
echo "finished distkeygen at $NOW"
echo "finished distkeygen at $NOW" >> $REPORT

# portfolio
NOW=`date`
echo "starting portfolio at $NOW"
echo "starting portfolio at $NOW" >> $REPORT
cd portfolio
echo "output from portfolio test" >> $STDOUTFILE
./start_portfolio.sh >> $STDOUTFILE 2>/tmp/errorportfolio.txt
cd ..
NOW=`date`
cat /tmp/errorportfolio.txt
echo "finished portfolio at $NOW"
echo "finished portfolio at $NOW" >> $REPORT

# eatmem
NOW=`date`
echo "starting eatmem at $NOW"
echo "starting eatmem at $NOW" >> $REPORT
cd eatmem
echo "output from eatmem test" >> $STDOUTFILE
./start_eatmem.sh >> $STDOUTFILE 2>/tmp/erroreatmem.txt
cd ..
NOW=`date`
cat /tmp/erroreatmem.txt
echo "finished eatmem at $NOW"
echo "finished eatmem at $NOW" >> $REPORT

# forkit
NOW=`date`
echo "starting forkit at $NOW"
echo "starting forkit at $NOW" >> $REPORT
cd fork
echo "output from forkit test" >> $STDOUTFILE
./start_forkit.sh >> $STDOUTFILE 2>/tmp/errorforkit.txt
cd ..
NOW=`date`
cat /tmp/errorforkit.txt
echo "finished forkit at $NOW"
echo "finished forkit at $NOW" >> $REPORT

# timewaster
NOW=`date`
echo "starting timewaster at $NOW"
echo "starting timewaster at $NOW" >> $REPORT
cd timewaster
echo "output from timewaster test" >> $STDOUTFILE
./start_timewaster.sh >> $STDOUTFILE 2>/tmp/errortimewaster.txt
cd ..
NOW=`date`
cat /tmp/errortimewaster.txt
echo "finished timewaster at $NOW"
echo "finished timewaster at $NOW" >> $REPORT



if [ $OMFS == 1 ]
 then
 # mfstest
 NOW=`date`
 echo "starting oMFStest at $NOW"
 echo "starting oMFStest at $NOW" >> $REPORT
 cd mfstest
 echo "output from oMFS test" >> $STDOUTFILE
 ./start_mfstest.sh >> $STDOUTFILE 2>/tmp/erroromfstest.txt
 cd ..
 NOW=`date`
 cat /tmp/erroromfstest.txt
 echo "finished oMFStest at $NOW"
 echo "finished oMFStest at $NOW" >> $REPORT
 else
 echo "oMFS was not tested" >> $REPORT
fi

#cleaning up
rm -f eatmem/out
kill $MOVING
cat /tmp/errormoving.txt

echo "output from the (local) kernel tests" >> $REPORT
echo "" >> $REPORT

# running the ltp kernel+syscall test
echo "running kernel+syscall on the local kernel"
echo "running kernel+syscall on the local kernel" >> $REPORT
echo "output from the (local) kernel tests" >> $REPORT
echo "" >> $REPORT
cd ltp
./start_ltp.sh >> $REPORT 2>/tmp/errorltp.txt
cd ..

cat /tmp/errorltp.txt
COUNTMIGS=`cat /proc/$MYSELF/nmigs`

echo "" >> $REPORT
echo "These are the results from the kernel-syscall test :" >> $REPORT
echo "" >> $REPORT
cat /tmp/kernel_syscall_test.txt >> $REPORT
echo "" >> $REPORT
echo "" >> $REPORT

echo "" >> $REPORT
echo "Here is output from the openMosix stress-test :" >> $REPORT
echo "" >> $REPORT
cat $STDOUTFILE >> $REPORT
echo "" >> $REPORT
echo "" >> $REPORT

echo "" >> $REPORT
echo "Here are the errors (stderr) which happens during the openMosix stress-test :" >> $REPORT
echo "" >> $REPORT
echo "Errors from the moving-test:" >> $REPORT
cat /tmp/errormoving.txt >> $REPORT
echo "Errors from the distkeygen-test:" >> $REPORT
cat /tmp/errordistkeygen.txt >> $REPORT
echo "Errors from the portfolio-test:" >> $REPORT
cat /tmp/errorportfolio.txt >> $REPORT
echo "Errors from the eatmem-test:" >> $REPORT
cat /tmp/erroreatmem.txt >> $REPORT
echo "Errors from the forkit-test:" >> $REPORT
cat /tmp/errorforkit.txt >> $REPORT
echo "Errors from the timewaster-test:" >> $REPORT
cat /tmp/errortimewaster.txt >> $REPORT

if [ $OMFS == 1 ]
 then
 echo "Errors from the oMFS-test:" >> $REPORT
 cat /tmp/erroromfstest.txt >> $REPORT
fi
echo "Errors from the kernel+syscall-test:" >> $REPORT
cat /tmp/errorltp.txt >> $REPORT
echo "" >> $REPORT
echo "" >> $REPORT
# cleaning up
/bin/rm -f /tmp/errormoving.txt /tmp/errordistkeygen.txt /tmp/errorportfolio.txt /tmp/erroreatmem.txt /tmp/errorforkit.txt /tmp/errortimewaster.txt /tmp/erroromfstest.txt /tmp/errorltp.txt $STDOUTFILE /tmp/kernel_syscall_test.txt /tmp/pslist.txt

echo "During the test-run the stress-test script itself was"  >> $REPORT
echo "migrated $COUNTMIGS times across your cluster"  >> $REPORT
echo "" >> $REPORT
echo "Use :" >> $REPORT
echo "" >> $REPORT
echo "grep 'stat=' $REPORT | grep -v 'stat=0' " >> $REPORT
echo "" >> $REPORT
echo "to view the syscall-test which passed NOT ok!" >> $REPORT


echo "" >> $REPORT
echo "(find more informations about this output at:" >> $REPORT
echo "http://ltp.sf.net)" >> $REPORT
echo "" >> $REPORT
echo "" >> $REPORT


echo
echo
echo "openMosix stress test finished"
echo "The results are saved in $REPORT"
echo




